@@ -12,8 +12,8 @@
 #include "nsCRT.h"
 #include "nsString.h"
 #include "mozilla/ArrayUtils.h"
-#include "mozilla/Preferences.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_accessibility.h"
 
 #include "nsControllerCommandTable.h"
 
@@ -122,10 +122,9 @@ class nsSelectionCommandsBase : public nsIControllerCommand {
   NS_IMETHOD GetCommandStateParams(const char* aCommandName,
                                    nsICommandParams* aParams,
                                    nsISupports* aCommandContext) override;
-  MOZ_CAN_RUN_SCRIPT
-  NS_IMETHOD DoCommandParams(const char* aCommandName,
-                             nsICommandParams* aParams,
-                             nsISupports* aCommandContext) override;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD
+  DoCommandParams(const char* aCommandName, nsICommandParams* aParams,
+                  nsISupports* aCommandContext) override;
 
  protected:
   virtual ~nsSelectionCommandsBase() = default;
@@ -142,8 +141,8 @@ class nsSelectionCommandsBase : public nsIControllerCommand {
 // caret' setting
 class nsSelectMoveScrollCommand : public nsSelectionCommandsBase {
  public:
-  NS_IMETHOD DoCommand(const char* aCommandName,
-                       nsISupports* aCommandContext) override;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD
+  DoCommand(const char* aCommandName, nsISupports* aCommandContext) override;
 
   // no member variables, please, we're stateless!
 };
@@ -151,8 +150,8 @@ class nsSelectMoveScrollCommand : public nsSelectionCommandsBase {
 // this class implements physical-movement versions of the above
 class nsPhysicalSelectMoveScrollCommand : public nsSelectionCommandsBase {
  public:
-  NS_IMETHOD DoCommand(const char* aCommandName,
-                       nsISupports* aCommandContext) override;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD
+  DoCommand(const char* aCommandName, nsISupports* aCommandContext) override;
 
   // no member variables, please, we're stateless!
 };
@@ -160,8 +159,8 @@ class nsPhysicalSelectMoveScrollCommand : public nsSelectionCommandsBase {
 // this class implements other selection commands
 class nsSelectCommand : public nsSelectionCommandsBase {
  public:
-  NS_IMETHOD DoCommand(const char* aCommandName,
-                       nsISupports* aCommandContext) override;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD
+  DoCommand(const char* aCommandName, nsISupports* aCommandContext) override;
 
   // no member variables, please, we're stateless!
 };
@@ -169,8 +168,8 @@ class nsSelectCommand : public nsSelectionCommandsBase {
 // this class implements physical-movement versions of selection commands
 class nsPhysicalSelectCommand : public nsSelectionCommandsBase {
  public:
-  NS_IMETHOD DoCommand(const char* aCommandName,
-                       nsISupports* aCommandContext) override;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD
+  DoCommand(const char* aCommandName, nsISupports* aCommandContext) override;
 
   // no member variables, please, we're stateless!
 };
@@ -255,7 +254,7 @@ static bool IsCaretOnInWindow(nsPIDOMWindowOuter* aWindow,
   bool caretOn = false;
   aSelCont->GetCaretEnabled(&caretOn);
   if (!caretOn) {
-    caretOn = Preferences::GetBool("accessibility.browsewithcaret");
+    caretOn = StaticPrefs::accessibility_browsewithcaret();
     if (caretOn) {
       nsCOMPtr<nsIDocShell> docShell = aWindow->GetDocShell();
       if (docShell && docShell->ItemType() == nsIDocShellTreeItem::typeChrome) {
@@ -577,8 +576,8 @@ nsresult nsClipboardCommand::DoCommand(const char* aCommandName,
 
   bool actionTaken = false;
   nsCopySupport::FireClipboardEvent(eventMessage,
-                                    nsIClipboard::kGlobalClipboard, presShell,
-                                    nullptr, &actionTaken);
+                                    Some(nsIClipboard::kGlobalClipboard),
+                                    presShell, nullptr, nullptr, &actionTaken);
 
   return actionTaken ? NS_OK : NS_SUCCESS_DOM_NO_OPERATION;
 }
