@@ -8,12 +8,7 @@
 
 "use strict";
 
-const OUTER_BASE_1 = "https://example.org/browser/dom/events/test/";
-const OUTER_BASE_2 = "https://example.org/browser/dom/events/test/";
-
-// iframe domains
-const INNER_BASE_1 = OUTER_BASE_1;
-const INNER_BASE_2 = OUTER_BASE_2;
+const kBaseUrl = "https://example.org/browser/dom/events/test/";
 
 add_setup(async function () {
   await SpecialPowers.pushPrefEnv({
@@ -23,7 +18,12 @@ add_setup(async function () {
     SpecialPowers.popPrefEnv();
   });
 
-  await setup();
+  await setup({
+    outerURL1: kBaseUrl + "browser_dragdrop_outer.html",
+    outerURL2: kBaseUrl + "browser_dragdrop_outer.html",
+    innerURL1: kBaseUrl + "browser_dragdrop_inner.html",
+    innerURL2: kBaseUrl + "browser_dragdrop_inner.html",
+  });
 });
 
 Services.scriptloader.loadSubScript(
