@@ -101,12 +101,15 @@ static bool IsWindowAllowedToPlayByTraits(nsPIDOMWindowInner* aWindow) {
     return false;
   }
 
+#ifndef MOZ_WIDGET_ANDROID
+  // On Android, we'd like to prevent top level video document from autoplaying.
   bool isTopLevelContent = !aWindow->GetBrowsingContext()->GetParent();
   if (currentDoc->MediaDocumentKind() == Document::MediaDocumentKind::Video &&
       isTopLevelContent) {
     AUTOPLAY_LOG("Allow top-level video document to autoplay.");
     return true;
   }
+#endif
 
   if (StaticPrefs::media_autoplay_allow_extension_background_pages() &&
       currentDoc->IsExtensionPage()) {
@@ -114,6 +117,12 @@ static bool IsWindowAllowedToPlayByTraits(nsPIDOMWindowInner* aWindow) {
     return true;
   }
 
+  if (currentDoc->GetPrincipal()->Equals(
+          nsContentUtils::GetFingerprintingProtectionPrincipal())) {
+    AUTOPLAY_LOG("Allow autoplay as in fingerprinting protection document.");
+    return true;
+  }
+
   return false;
 }
 
@@ -148,12 +157,6 @@ static bool IsMediaElementInaudible(const HTMLMediaElement& aElement) {
   return false;
 }
 
-static bool IsAudioContextAllowedToPlay(const AudioContext& aContext) {
-  // Offline context won't directly output sound to audio devices.
-  return aContext.IsOffline() ||
-         IsWindowAllowedToPlayOverall(aContext.GetParentObject());
-}
-
 static bool IsEnableBlockingWebAudioByUserGesturePolicy() {
   return StaticPrefs::media_autoplay_blocking_policy() ==
          sPOLICY_STICKY_ACTIVATION;
@@ -297,7 +300,7 @@ bool AutoplayPolicy::IsAllowedToPlay(const AudioContext& aContext) {
     return true;
   }
 
-  nsPIDOMWindowInner* window = aContext.GetParentObject();
+  nsPIDOMWindowInner* window = aContext.GetOwnerWindow();
   uint32_t sitePermission = SiteAutoplayPerm(window);
 
   if (sitePermission == nsIPermissionManager::ALLOW_ACTION) {
@@ -395,12 +398,6 @@ uint32_t AutoplayPolicy::GetSiteAutoplayPermission(nsIPrincipal* aPrincipal) {
   return perm;
 }
 
-/* static */
-bool AutoplayPolicyTelemetryUtils::WouldBeAllowedToPlayIfAutoplayDisabled(
-    const AudioContext& aContext) {
-  return IsAudioContextAllowedToPlay(aContext);
-}
-
 /* static */
 dom::AutoplayPolicy AutoplayPolicy::GetAutoplayPolicy(
     const dom::HTMLMediaElement& aElement) {
