@@ -6,6 +6,7 @@
 #include "gtest/gtest.h"
 
 #include "AnnexB.h"
+#include "BufferReader.h"
 #include "H264.h"
 #include "ImageContainer.h"
 #include "mozilla/AbstractThread.h"
@@ -18,6 +19,29 @@
 #include "VPXDecoder.h"
 #include <algorithm>
 
+#ifdef MOZ_WIDGET_ANDROID
+// Create/init a H.264 encoder and check if it's SW.
+#  define SKIP_IF_ANDROID_SW()                                           \
+    do {                                                                 \
+      RefPtr<MediaDataEncoder> e = CreateH264Encoder(                    \
+          Usage::Record,                                                 \
+          EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),  \
+          kImageSize, ScalabilityMode::None, Some(kH264SpecificAnnexB)); \
+      if (EnsureInit(e)) {                                               \
+        nsCString dummy;                                                 \
+        bool isSW = !e->IsHardwareAccelerated(dummy);                    \
+        WaitForShutdown(e);                                              \
+        if (isSW) {                                                      \
+          return;                                                        \
+        }                                                                \
+      }                                                                  \
+    } while (0)
+#else
+#  define SKIP_IF_ANDROID_SW() \
+    do {                       \
+    } while (0)
+#endif
+
 #define RUN_IF_SUPPORTED(codecType, test)   \
   do {                                      \
     RefPtr<PEMFactory> f(new PEMFactory()); \
@@ -27,8 +51,6 @@
   } while (0)
 
 #define BLOCK_SIZE 64
-#define WIDTH 640
-#define HEIGHT 480
 #define NUM_FRAMES 150UL
 #define FRAME_RATE 30
 #define FRAME_DURATION (1000000 / FRAME_RATE)
@@ -38,12 +60,15 @@
 
 using namespace mozilla;
 
-static gfx::IntSize kImageSize(WIDTH, HEIGHT);
+static gfx::IntSize kImageSize(640, 480);
+static gfx::IntSize kImageSize4K(3840, 2160);
 // Set codec to avc1.42001E - Base profile, constraint 0, level 30.
-const H264Specific kH264SpecificAnnexB(H264_PROFILE_BASE, H264_LEVEL_3,
-                                       H264BitStreamFormat::ANNEXB);
-const H264Specific kH264SpecificAVCC(H264_PROFILE_BASE, H264_LEVEL_3,
-                                     H264BitStreamFormat::AVC);
+MOZ_RUNINIT const H264Specific kH264SpecificAnnexB(H264_PROFILE_BASE,
+                                                   H264_LEVEL::H264_LEVEL_3,
+                                                   H264BitStreamFormat::ANNEXB);
+MOZ_RUNINIT const H264Specific kH264SpecificAVCC(H264_PROFILE_BASE,
+                                                 H264_LEVEL::H264_LEVEL_3,
+                                                 H264BitStreamFormat::AVC);
 
 class MediaDataEncoderTest : public testing::Test {
  protected:
@@ -51,9 +76,13 @@ class MediaDataEncoderTest : public testing::Test {
     Preferences::SetBool("media.ffmpeg.encoder.enabled", true);
     Preferences::SetInt("logging.FFmpegVideo", 5);
     mData.Init(kImageSize);
+    mData4K.Init(kImageSize4K);
   }
 
-  void TearDown() override { mData.Deinit(); }
+  void TearDown() override {
+    mData.Deinit();
+    mData4K.Deinit();
+  }
 
  public:
   struct FrameSource final {
@@ -152,12 +181,13 @@ class MediaDataEncoderTest : public testing::Test {
 
  public:
   FrameSource mData;
+  FrameSource mData4K;
 };
 
 template <typename T>
 already_AddRefed<MediaDataEncoder> CreateVideoEncoder(
-    CodecType aCodec, Usage aUsage, dom::ImageBitmapFormat aPixelFormat,
-    int32_t aWidth, int32_t aHeight, ScalabilityMode aScalabilityMode,
+    CodecType aCodec, Usage aUsage, EncoderConfig::SampleFormat aFormat,
+    gfx::IntSize aSize, ScalabilityMode aScalabilityMode,
     const Maybe<T>& aSpecific) {
   RefPtr<PEMFactory> f(new PEMFactory());
 
@@ -172,8 +202,7 @@ already_AddRefed<MediaDataEncoder> CreateVideoEncoder(
   RefPtr<MediaDataEncoder> e;
   const HardwarePreference pref = HardwarePreference::None;
   e = f->CreateEncoder(
-      EncoderConfig(aCodec, gfx::IntSize{aWidth, aHeight}, aUsage, aPixelFormat,
-                    aPixelFormat, FRAME_RATE /* FPS */,
+      EncoderConfig(aCodec, aSize, aUsage, aFormat, FRAME_RATE /* FPS */,
                     KEYFRAME_INTERVAL /* keyframe interval */,
                     BIT_RATE /* bitrate */, 0, 0, BIT_RATE_MODE, pref,
                     aScalabilityMode, aSpecific),
@@ -184,12 +213,13 @@ already_AddRefed<MediaDataEncoder> CreateVideoEncoder(
 
 static already_AddRefed<MediaDataEncoder> CreateH264Encoder(
     Usage aUsage = Usage::Realtime,
-    dom::ImageBitmapFormat aPixelFormat = dom::ImageBitmapFormat::YUV420P,
-    int32_t aWidth = WIDTH, int32_t aHeight = HEIGHT,
+    EncoderConfig::SampleFormat aFormat =
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+    gfx::IntSize aSize = kImageSize,
     ScalabilityMode aScalabilityMode = ScalabilityMode::None,
     const Maybe<H264Specific>& aSpecific = Some(kH264SpecificAnnexB)) {
-  return CreateVideoEncoder(CodecType::H264, aUsage, aPixelFormat, aWidth,
-                            aHeight, aScalabilityMode, aSpecific);
+  return CreateVideoEncoder(CodecType::H264, aUsage, aFormat, aSize,
+                            aScalabilityMode, aSpecific);
 }
 
 void WaitForShutdown(const RefPtr<MediaDataEncoder>& aEncoder) {
@@ -233,9 +263,10 @@ static bool EnsureInit(const RefPtr<MediaDataEncoder>& aEncoder) {
 TEST_F(MediaDataEncoderTest, H264Inits) {
   RUN_IF_SUPPORTED(CodecType::H264, []() {
     // w/o codec specific: should fail for h264.
-    RefPtr<MediaDataEncoder> e =
-        CreateH264Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                          WIDTH, HEIGHT, ScalabilityMode::None, Nothing());
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Nothing());
     EXPECT_FALSE(e);
 
     // w/ codec specific
@@ -285,10 +316,47 @@ static MediaDataEncoder::EncodedData Encode(
   return output;
 }
 
-TEST_F(MediaDataEncoderTest, H264Encodes) {
+Result<uint8_t, nsresult> GetNALUSize(const mozilla::MediaRawData* aSample) {
+  return AVCCConfig::Parse(aSample).map(
+      [](AVCCConfig config) { return config.NALUSize(); });
+}
+
+Result<Ok, nsresult> IsValidAVCC(const mozilla::MediaRawData* aSample,
+                                 uint8_t aNALUSize) {
+  BufferReader reader(aSample->Data(), aSample->Size());
+  while (reader.Remaining() >= aNALUSize) {
+    uint32_t nalLen;
+    switch (aNALUSize) {
+      case 1:
+        MOZ_TRY_VAR(nalLen, reader.ReadU8());
+        break;
+      case 2:
+        MOZ_TRY_VAR(nalLen, reader.ReadU16());
+        break;
+      case 3:
+        MOZ_TRY_VAR(nalLen, reader.ReadU24());
+        break;
+      case 4:
+        MOZ_TRY_VAR(nalLen, reader.ReadU32());
+        break;
+      default:
+        return Err(NS_ERROR_INVALID_ARG);
+    }
+    const uint8_t* p = reader.Read(nalLen);
+    if (!p) {
+      return Err(NS_ERROR_ILLEGAL_VALUE);
+    }
+  }
+  return Ok();
+}
+
+TEST_F(MediaDataEncoderTest, H264EncodesAnnexBRecord) {
   RUN_IF_SUPPORTED(CodecType::H264, [this]() {
     // Encode one frame and output in AnnexB format.
-    RefPtr<MediaDataEncoder> e = CreateH264Encoder();
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EnsureInit(e);
     MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData);
     EXPECT_EQ(output.Length(), 1UL);
@@ -296,7 +364,10 @@ TEST_F(MediaDataEncoderTest, H264Encodes) {
     WaitForShutdown(e);
 
     // Encode multiple frames and output in AnnexB format.
-    e = CreateH264Encoder();
+    e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EnsureInit(e);
     output = Encode(e, NUM_FRAMES, mData);
     EXPECT_EQ(output.Length(), NUM_FRAMES);
@@ -304,17 +375,242 @@ TEST_F(MediaDataEncoderTest, H264Encodes) {
       EXPECT_TRUE(AnnexB::IsAnnexB(frame));
     }
     WaitForShutdown(e);
+  });
+}
 
-    // Encode one frame and output in avcC format.
-    e = CreateH264Encoder(Usage::Record, dom::ImageBitmapFormat::YUV420P, WIDTH,
-                          HEIGHT, ScalabilityMode::None,
-                          Some(kH264SpecificAVCC));
+TEST_F(MediaDataEncoderTest, H264EncodesAnnexBRealtime) {
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in AnnexB format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAnnexB(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in AnnexB format.
+    e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EnsureInit(e);
     output = Encode(e, NUM_FRAMES, mData);
     EXPECT_EQ(output.Length(), NUM_FRAMES);
-    AnnexB::IsAVCC(output[0]);  // Only 1st frame has extra data.
     for (auto frame : output) {
-      EXPECT_FALSE(AnnexB::IsAnnexB(frame));
+      EXPECT_TRUE(AnnexB::IsAnnexB(frame));
+    }
+    WaitForShutdown(e);
+  });
+}
+
+TEST_F(MediaDataEncoderTest, H264EncodesAVCCRecord) {
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in avcC format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in avcC format.
+    e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    output = Encode(e, NUM_FRAMES, mData);
+    EXPECT_EQ(output.Length(), NUM_FRAMES);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    uint8_t naluSize = GetNALUSize(output[0]).unwrapOr(0);
+    EXPECT_GT(naluSize, 0);
+    EXPECT_LE(naluSize, 4);
+    for (auto frame : output) {
+      if (frame->mExtraData && !frame->mExtraData->IsEmpty()) {
+        naluSize = GetNALUSize(frame).unwrapOr(0);
+        EXPECT_GT(naluSize, 0);
+        EXPECT_LE(naluSize, 4);
+      }
+      EXPECT_TRUE(IsValidAVCC(frame, naluSize).isOk());
+    }
+    WaitForShutdown(e);
+  });
+}
+
+TEST_F(MediaDataEncoderTest, H264EncodesAVCCRealtime) {
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in avcC format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in avcC format.
+    e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    output = Encode(e, NUM_FRAMES, mData);
+    EXPECT_EQ(output.Length(), NUM_FRAMES);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    uint8_t naluSize = GetNALUSize(output[0]).unwrapOr(0);
+    EXPECT_GT(naluSize, 0);
+    EXPECT_LE(naluSize, 4);
+    for (auto frame : output) {
+      if (frame->mExtraData && !frame->mExtraData->IsEmpty()) {
+        naluSize = GetNALUSize(frame).unwrapOr(0);
+        EXPECT_GT(naluSize, 0);
+        EXPECT_LE(naluSize, 4);
+      }
+      EXPECT_TRUE(IsValidAVCC(frame, naluSize).isOk());
+    }
+    WaitForShutdown(e);
+  });
+}
+
+TEST_F(MediaDataEncoderTest, H264Encodes4KAnnexBRecord) {
+  SKIP_IF_ANDROID_SW();  // Android SW can't encode 4K.
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in AnnexB format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData4K);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAnnexB(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in AnnexB format.
+    e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    EnsureInit(e);
+    output = Encode(e, NUM_FRAMES, mData4K);
+    EXPECT_EQ(output.Length(), NUM_FRAMES);
+    for (auto frame : output) {
+      EXPECT_TRUE(AnnexB::IsAnnexB(frame));
+    }
+    WaitForShutdown(e);
+  });
+}
+
+TEST_F(MediaDataEncoderTest, H264Encodes4KAnnexBRealtime) {
+  SKIP_IF_ANDROID_SW();  // Android SW can't encode 4K.
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in AnnexB format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData4K);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAnnexB(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in AnnexB format.
+    e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    EnsureInit(e);
+    output = Encode(e, NUM_FRAMES, mData4K);
+    EXPECT_LE(output.Length(),
+              NUM_FRAMES);  // Frames can be dropped in realtime usage.
+    for (auto frame : output) {
+      EXPECT_TRUE(AnnexB::IsAnnexB(frame));
+    }
+    WaitForShutdown(e);
+  });
+}
+
+TEST_F(MediaDataEncoderTest, H264Encodes4KAVCCRecord) {
+  SKIP_IF_ANDROID_SW();  // Android SW can't encode 4K.
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in avcC format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData4K);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in avcC format.
+    e = CreateH264Encoder(
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    output = Encode(e, NUM_FRAMES, mData4K);
+    EXPECT_EQ(output.Length(), NUM_FRAMES);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    uint8_t naluSize = GetNALUSize(output[0]).unwrapOr(0);
+    EXPECT_GT(naluSize, 0);
+    EXPECT_LE(naluSize, 4);
+    for (auto frame : output) {
+      if (frame->mExtraData && !frame->mExtraData->IsEmpty()) {
+        naluSize = GetNALUSize(frame).unwrapOr(0);
+        EXPECT_GT(naluSize, 0);
+        EXPECT_LE(naluSize, 4);
+      }
+      EXPECT_TRUE(IsValidAVCC(frame, naluSize).isOk());
+    }
+    WaitForShutdown(e);
+  });
+}
+
+TEST_F(MediaDataEncoderTest, H264Encodes4KAVCCRealtime) {
+  SKIP_IF_ANDROID_SW();  // Android SW can't encode 4K.
+  RUN_IF_SUPPORTED(CodecType::H264, [this]() {
+    // Encode one frame and output in avcC format.
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    MediaDataEncoder::EncodedData output = Encode(e, 1UL, mData4K);
+    EXPECT_EQ(output.Length(), 1UL);
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    WaitForShutdown(e);
+
+    // Encode multiple frames and output in avcC format.
+    e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize4K, ScalabilityMode::None, Some(kH264SpecificAVCC));
+    EnsureInit(e);
+    output = Encode(e, NUM_FRAMES, mData4K);
+    EXPECT_LE(output.Length(),
+              NUM_FRAMES);  // Frames can be dropped in realtime usage.
+    EXPECT_TRUE(AnnexB::IsAVCC(output[0]));
+    uint8_t naluSize = GetNALUSize(output[0]).unwrapOr(0);
+    EXPECT_GT(naluSize, 0);
+    EXPECT_LE(naluSize, 4);
+    for (auto frame : output) {
+      if (frame->mExtraData && !frame->mExtraData->IsEmpty()) {
+        naluSize = GetNALUSize(frame).unwrapOr(0);
+        EXPECT_GT(naluSize, 0);
+        EXPECT_LE(naluSize, 4);
+      }
+      EXPECT_TRUE(IsValidAVCC(frame, naluSize).isOk());
     }
     WaitForShutdown(e);
   });
@@ -335,56 +631,67 @@ TEST_F(MediaDataEncoderTest, H264Duration) {
 
 TEST_F(MediaDataEncoderTest, InvalidSize) {
   RUN_IF_SUPPORTED(CodecType::H264, []() {
-    RefPtr<MediaDataEncoder> e0x0 =
-        CreateH264Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P, 0,
-                          0, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    RefPtr<MediaDataEncoder> e0x0 = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P), {0, 0},
+        ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EXPECT_EQ(e0x0, nullptr);
 
-    RefPtr<MediaDataEncoder> e0x1 =
-        CreateH264Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P, 0,
-                          1, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    RefPtr<MediaDataEncoder> e0x1 = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P), {0, 1},
+        ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EXPECT_EQ(e0x1, nullptr);
 
-    RefPtr<MediaDataEncoder> e1x0 =
-        CreateH264Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P, 1,
-                          0, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    RefPtr<MediaDataEncoder> e1x0 = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P), {1, 0},
+        ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EXPECT_EQ(e1x0, nullptr);
   });
 }
 
+// For Android HW encoder only.
 #ifdef MOZ_WIDGET_ANDROID
 TEST_F(MediaDataEncoderTest, AndroidNotSupportedSize) {
+  SKIP_IF_ANDROID_SW();
   RUN_IF_SUPPORTED(CodecType::H264, []() {
-    RefPtr<MediaDataEncoder> e =
-        CreateH264Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P, 1,
-                          1, ScalabilityMode::None, Some(kH264SpecificAnnexB));
+    RefPtr<MediaDataEncoder> e = CreateH264Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P), {1, 1},
+        ScalabilityMode::None, Some(kH264SpecificAnnexB));
     EXPECT_NE(e, nullptr);
     EXPECT_FALSE(EnsureInit(e));
   });
 }
 #endif
 
-#if defined(XP_LINUX) && !defined(ANDROID)
+#if !defined(ANDROID)
 TEST_F(MediaDataEncoderTest, H264AVCC) {
   RUN_IF_SUPPORTED(CodecType::H264, [this]() {
     // Encod frames in avcC format.
     RefPtr<MediaDataEncoder> e = CreateH264Encoder(
-        Usage::Record, dom::ImageBitmapFormat::YUV420P, WIDTH, HEIGHT,
-        ScalabilityMode::None, Some(kH264SpecificAVCC));
+        Usage::Record,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Some(kH264SpecificAVCC));
     EnsureInit(e);
     MediaDataEncoder::EncodedData output = Encode(e, NUM_FRAMES, mData);
     EXPECT_EQ(output.Length(), NUM_FRAMES);
     for (auto frame : output) {
       EXPECT_FALSE(AnnexB::IsAnnexB(frame));
       if (frame->mKeyframe) {
-        AnnexB::IsAVCC(frame);
-        AVCCConfig config = AVCCConfig::Parse(frame).unwrap();
-        EXPECT_EQ(config.mAVCProfileIndication,
-                  static_cast<decltype(config.mAVCProfileIndication)>(
-                      kH264SpecificAVCC.mProfile));
-        EXPECT_EQ(config.mAVCLevelIndication,
-                  static_cast<decltype(config.mAVCLevelIndication)>(
-                      kH264SpecificAVCC.mLevel));
+        // The extradata may be included at the beginning, whenever it changes,
+        // or with every keyframe to support robust seeking or decoder resets.
+        if (frame->mExtraData && !frame->mExtraData->IsEmpty()) {
+          EXPECT_TRUE(AnnexB::IsAVCC(frame));
+          AVCCConfig config = AVCCConfig::Parse(frame).unwrap();
+          EXPECT_EQ(config.mAVCProfileIndication,
+                    static_cast<decltype(config.mAVCProfileIndication)>(
+                        kH264SpecificAVCC.mProfile));
+          EXPECT_EQ(config.mAVCLevelIndication,
+                    static_cast<decltype(config.mAVCLevelIndication)>(
+                        kH264SpecificAVCC.mLevel));
+        }
       }
     }
     WaitForShutdown(e);
@@ -394,22 +701,24 @@ TEST_F(MediaDataEncoderTest, H264AVCC) {
 
 static already_AddRefed<MediaDataEncoder> CreateVP8Encoder(
     Usage aUsage = Usage::Realtime,
-    dom::ImageBitmapFormat aPixelFormat = dom::ImageBitmapFormat::YUV420P,
-    int32_t aWidth = WIDTH, int32_t aHeight = HEIGHT,
+    EncoderConfig::SampleFormat aFormat =
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+    gfx::IntSize aSize = kImageSize,
     ScalabilityMode aScalabilityMode = ScalabilityMode::None,
     const Maybe<VP8Specific>& aSpecific = Some(VP8Specific())) {
-  return CreateVideoEncoder(CodecType::VP8, aUsage, aPixelFormat, aWidth,
-                            aHeight, aScalabilityMode, aSpecific);
+  return CreateVideoEncoder(CodecType::VP8, aUsage, aFormat, aSize,
+                            aScalabilityMode, aSpecific);
 }
 
 static already_AddRefed<MediaDataEncoder> CreateVP9Encoder(
     Usage aUsage = Usage::Realtime,
-    dom::ImageBitmapFormat aPixelFormat = dom::ImageBitmapFormat::YUV420P,
-    int32_t aWidth = WIDTH, int32_t aHeight = HEIGHT,
+    EncoderConfig::SampleFormat aFormat =
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+    gfx::IntSize aSize = kImageSize,
     ScalabilityMode aScalabilityMode = ScalabilityMode::None,
     const Maybe<VP9Specific>& aSpecific = Some(VP9Specific())) {
-  return CreateVideoEncoder(CodecType::VP9, aUsage, aPixelFormat, aWidth,
-                            aHeight, aScalabilityMode, aSpecific);
+  return CreateVideoEncoder(CodecType::VP9, aUsage, aFormat, aSize,
+                            aScalabilityMode, aSpecific);
 }
 
 TEST_F(MediaDataEncoderTest, VP8Create) {
@@ -423,9 +732,10 @@ TEST_F(MediaDataEncoderTest, VP8Create) {
 TEST_F(MediaDataEncoderTest, VP8Inits) {
   RUN_IF_SUPPORTED(CodecType::VP8, []() {
     // w/o codec specific.
-    RefPtr<MediaDataEncoder> e =
-        CreateVP8Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                         WIDTH, HEIGHT, ScalabilityMode::None, Nothing());
+    RefPtr<MediaDataEncoder> e = CreateVP8Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Nothing());
     EXPECT_TRUE(EnsureInit(e));
     WaitForShutdown(e);
 
@@ -483,7 +793,7 @@ TEST_F(MediaDataEncoderTest, VP8Duration) {
   });
 }
 
-#if defined(XP_LINUX) && !defined(ANDROID)
+#if !defined(ANDROID)
 TEST_F(MediaDataEncoderTest, VP8EncodeAfterDrain) {
   RUN_IF_SUPPORTED(CodecType::VP8, [this]() {
     RefPtr<MediaDataEncoder> e = CreateVP8Encoder();
@@ -527,19 +837,23 @@ TEST_F(MediaDataEncoderTest, VP8EncodeWithScalabilityModeL1T2) {
                          false,                 /* mAutoResize */
                          false                  /* mFrameDropping */
     );
-    RefPtr<MediaDataEncoder> e =
-        CreateVP8Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                         WIDTH, HEIGHT, ScalabilityMode::L1T2, Some(specific));
+    RefPtr<MediaDataEncoder> e = CreateVP8Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::L1T2, Some(specific));
     EnsureInit(e);
 
     const nsTArray<uint8_t> pattern({0, 1});
     MediaDataEncoder::EncodedData output = Encode(e, NUM_FRAMES, mData);
+    int temporal_idx = 0;
     EXPECT_EQ(output.Length(), NUM_FRAMES);
     for (size_t i = 0; i < output.Length(); ++i) {
       const RefPtr<MediaRawData> frame = output[i];
-
+      if (frame->mKeyframe) {
+        temporal_idx = 0;
+      }
       EXPECT_TRUE(frame->mTemporalLayerId);
-      size_t idx = i % pattern.Length();
+      size_t idx = temporal_idx++ % pattern.Length();
       EXPECT_EQ(frame->mTemporalLayerId.value(), pattern[idx]);
     }
     WaitForShutdown(e);
@@ -555,19 +869,23 @@ TEST_F(MediaDataEncoderTest, VP8EncodeWithScalabilityModeL1T3) {
                          false,                 /* mAutoResize */
                          false                  /* mFrameDropping */
     );
-    RefPtr<MediaDataEncoder> e =
-        CreateVP8Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                         WIDTH, HEIGHT, ScalabilityMode::L1T3, Some(specific));
+    RefPtr<MediaDataEncoder> e = CreateVP8Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::L1T3, Some(specific));
     EnsureInit(e);
 
     const nsTArray<uint8_t> pattern({0, 2, 1, 2});
     MediaDataEncoder::EncodedData output = Encode(e, NUM_FRAMES, mData);
     EXPECT_EQ(output.Length(), NUM_FRAMES);
+    int temporal_idx = 0;
     for (size_t i = 0; i < output.Length(); ++i) {
       const RefPtr<MediaRawData> frame = output[i];
-
+      if (frame->mKeyframe) {
+        temporal_idx = 0;
+      }
       EXPECT_TRUE(frame->mTemporalLayerId);
-      size_t idx = i % pattern.Length();
+      size_t idx = temporal_idx++ % pattern.Length();
       EXPECT_EQ(frame->mTemporalLayerId.value(), pattern[idx]);
     }
     WaitForShutdown(e);
@@ -586,9 +904,10 @@ TEST_F(MediaDataEncoderTest, VP9Create) {
 TEST_F(MediaDataEncoderTest, VP9Inits) {
   RUN_IF_SUPPORTED(CodecType::VP9, []() {
     // w/o codec specific.
-    RefPtr<MediaDataEncoder> e =
-        CreateVP9Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                         WIDTH, HEIGHT, ScalabilityMode::None, Nothing());
+    RefPtr<MediaDataEncoder> e = CreateVP9Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::None, Nothing());
     EXPECT_TRUE(EnsureInit(e));
     WaitForShutdown(e);
 
@@ -644,7 +963,7 @@ TEST_F(MediaDataEncoderTest, VP9Duration) {
   });
 }
 
-#if defined(XP_LINUX) && !defined(ANDROID)
+#if !defined(ANDROID)
 TEST_F(MediaDataEncoderTest, VP9EncodeAfterDrain) {
   RUN_IF_SUPPORTED(CodecType::VP9, [this]() {
     RefPtr<MediaDataEncoder> e = CreateVP9Encoder();
@@ -692,18 +1011,23 @@ TEST_F(MediaDataEncoderTest, VP9EncodeWithScalabilityModeL1T2) {
                          false                  /* mFlexible */
     );
 
-    RefPtr<MediaDataEncoder> e =
-        CreateVP9Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                         WIDTH, HEIGHT, ScalabilityMode::L1T2, Some(specific));
+    RefPtr<MediaDataEncoder> e = CreateVP9Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::L1T2, Some(specific));
     EnsureInit(e);
 
     const nsTArray<uint8_t> pattern({0, 1});
     MediaDataEncoder::EncodedData output = Encode(e, NUM_FRAMES, mData);
+    int temporal_idx = 0;
     EXPECT_EQ(output.Length(), NUM_FRAMES);
     for (size_t i = 0; i < output.Length(); ++i) {
       const RefPtr<MediaRawData> frame = output[i];
+      if (frame->mKeyframe) {
+        temporal_idx = 0;
+      }
       EXPECT_TRUE(frame->mTemporalLayerId);
-      size_t idx = i % pattern.Length();
+      size_t idx = temporal_idx++ % pattern.Length();
       EXPECT_EQ(frame->mTemporalLayerId.value(), pattern[idx]);
     }
     WaitForShutdown(e);
@@ -723,18 +1047,23 @@ TEST_F(MediaDataEncoderTest, VP9EncodeWithScalabilityModeL1T3) {
                          false                  /* mFlexible */
     );
 
-    RefPtr<MediaDataEncoder> e =
-        CreateVP9Encoder(Usage::Realtime, dom::ImageBitmapFormat::YUV420P,
-                         WIDTH, HEIGHT, ScalabilityMode::L1T3, Some(specific));
+    RefPtr<MediaDataEncoder> e = CreateVP9Encoder(
+        Usage::Realtime,
+        EncoderConfig::SampleFormat(dom::ImageBitmapFormat::YUV420P),
+        kImageSize, ScalabilityMode::L1T3, Some(specific));
     EnsureInit(e);
 
     const nsTArray<uint8_t> pattern({0, 2, 1, 2});
     MediaDataEncoder::EncodedData output = Encode(e, NUM_FRAMES, mData);
+    int temporal_idx = 0;
     EXPECT_EQ(output.Length(), NUM_FRAMES);
     for (size_t i = 0; i < output.Length(); ++i) {
       const RefPtr<MediaRawData> frame = output[i];
+      if (frame->mKeyframe) {
+        temporal_idx = 0;
+      }
       EXPECT_TRUE(frame->mTemporalLayerId);
-      size_t idx = i % pattern.Length();
+      size_t idx = temporal_idx++ % pattern.Length();
       EXPECT_EQ(frame->mTemporalLayerId.value(), pattern[idx]);
     }
     WaitForShutdown(e);
