@@ -8,6 +8,7 @@
 #include "CamerasChild.h"
 #include "MediaManager.h"
 #include "MediaTrackConstraints.h"
+#include "mozilla/dom/MediaTrackCapabilitiesBinding.h"
 #include "mozilla/dom/MediaTrackSettingsBinding.h"
 #include "mozilla/ErrorNames.h"
 #include "mozilla/gfx/Point.h"
@@ -29,6 +30,7 @@ extern LazyLogModule gMediaManagerLog;
 
 using dom::ConstrainLongRange;
 using dom::MediaSourceEnum;
+using dom::MediaTrackCapabilities;
 using dom::MediaTrackConstraints;
 using dom::MediaTrackConstraintSet;
 using dom::MediaTrackSettings;
@@ -66,12 +68,6 @@ static Maybe<VideoFacingModeEnum> GetFacingMode(const nsString& aDeviceName) {
     return Some(VideoFacingModeEnum::User);
   }
 #endif  // ANDROID
-#ifdef XP_MACOSX
-  // Kludge to test user-facing cameras on OSX.
-  if (aDeviceName.Find(u"Face"_ns) != -1) {
-    return Some(VideoFacingModeEnum::User);
-  }
-#endif
 #ifdef XP_WIN
   // The cameras' name of Surface book are "Microsoft Camera Front" and
   // "Microsoft Camera Rear" respectively.
@@ -96,6 +92,8 @@ MediaEngineRemoteVideoSource::MediaEngineRemoteVideoSource(
                            /* max_number_of_buffers */ 1),
       mSettingsUpdatedByFrame(MakeAndAddRef<media::Refcountable<AtomicBool>>()),
       mSettings(MakeAndAddRef<media::Refcountable<MediaTrackSettings>>()),
+      mTrackCapabilities(
+          MakeAndAddRef<media::Refcountable<MediaTrackCapabilities>>()),
       mFirstFramePromise(mFirstFramePromiseHolder.Ensure(__func__)),
       mMediaDevice(aMediaDevice),
       mDeviceUUID(NS_ConvertUTF16toUTF8(aMediaDevice->mRawID)) {
@@ -110,6 +108,9 @@ MediaEngineRemoteVideoSource::MediaEngineRemoteVideoSource(
     if (facingMode.isSome()) {
       NS_ConvertASCIItoUTF16 facingString(dom::GetEnumString(*facingMode));
       mSettings->mFacingMode.Construct(facingString);
+      nsTArray<nsString> facing;
+      facing.AppendElement(facingString);
+      mTrackCapabilities->mFacingMode.Construct(std::move(facing));
       mFacingMode.emplace(facingString);
     }
   }
@@ -216,7 +217,7 @@ nsresult MediaEngineRemoteVideoSource::Start() {
   LOG("%s", __PRETTY_FUNCTION__);
   AssertIsOnOwningThread();
 
-  MOZ_ASSERT(mState == kAllocated || mState == kStopped);
+  MOZ_ASSERT(mState == kAllocated || mState == kStarted || mState == kStopped);
   MOZ_ASSERT(mTrack);
 
   {
@@ -317,26 +318,13 @@ nsresult MediaEngineRemoteVideoSource::Reconfigure(
     return NS_OK;
   }
 
-  bool started = mState == kStarted;
-  if (started) {
-    nsresult rv = Stop();
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      nsAutoCString name;
-      GetErrorName(rv, name);
-      LOG("Video source %p for video device %d Reconfigure() failed "
-          "unexpectedly in Stop(). rv=%s",
-          this, mCaptureId, name.Data());
-      return NS_ERROR_UNEXPECTED;
-    }
-  }
-
   {
     MutexAutoLock lock(mMutex);
     // Start() applies mCapability on the device.
     mCapability = newCapability;
   }
 
-  if (started) {
+  if (mState == kStarted) {
     nsresult rv = Start();
     if (NS_WARN_IF(NS_FAILED(rv))) {
       nsAutoCString name;
@@ -393,6 +381,7 @@ const TrackingId& MediaEngineRemoteVideoSource::GetTrackingId() const {
 }
 
 void MediaEngineRemoteVideoSource::OnCaptureEnded() {
+  mFirstFramePromiseHolder.RejectIfExists(NS_ERROR_UNEXPECTED, __func__);
   mCaptureEndedEvent.Notify();
 }
 
@@ -542,10 +531,10 @@ int MediaEngineRemoteVideoSource::DeliverFrame(
 #ifdef DEBUG
   static uint32_t frame_num = 0;
   LOG_FRAME(
-      "frame %d (%dx%d)->(%dx%d); rotation %d, timeStamp %u, ntpTimeMs %" PRIu64
-      ", renderTimeMs %" PRIu64,
+      "frame %d (%dx%d)->(%dx%d); rotation %d, rtpTimeStamp %u, ntpTimeMs "
+      "%" PRIu64 ", renderTimeMs %" PRIu64,
       frame_num++, aProps.width(), aProps.height(), dst_width, dst_height,
-      aProps.rotation(), aProps.timeStamp(), aProps.ntpTimeMs(),
+      aProps.rotation(), aProps.rtpTimeStamp(), aProps.ntpTimeMs(),
       aProps.renderTimeMs());
 #endif
 
@@ -571,7 +560,9 @@ int MediaEngineRemoteVideoSource::DeliverFrame(
     MOZ_ASSERT(mState == kStarted);
     VideoSegment segment;
     mImageSize = image->GetSize();
-    segment.AppendFrame(image.forget(), mImageSize, mPrincipal);
+    segment.AppendWebrtcLocalFrame(image.forget(), mImageSize, mPrincipal,
+                                   /* aForceBlack */ false, TimeStamp::Now(),
+                                   aProps.captureTime());
     mTrack->AppendData(&segment);
   }
 
@@ -782,10 +773,50 @@ bool MediaEngineRemoteVideoSource::ChooseCapability(
 
   nsTArray<CapabilityCandidate> candidateSet;
   size_t num = NumCapabilities();
+  int32_t minHeight = 0, maxHeight = 0, minWidth = 0, maxWidth = 0, maxFps = 0;
   for (size_t i = 0; i < num; i++) {
-    candidateSet.AppendElement(CapabilityCandidate(GetCapability(i)));
+    auto capability = GetCapability(i);
+    if (capability.height > maxHeight) {
+      maxHeight = capability.height;
+    }
+    if (!minHeight || (capability.height < minHeight)) {
+      minHeight = capability.height;
+    }
+    if (capability.width > maxWidth) {
+      maxWidth = capability.width;
+    }
+    if (!minWidth || (capability.width < minWidth)) {
+      minWidth = capability.width;
+    }
+    if (capability.maxFPS > maxFps) {
+      maxFps = capability.maxFPS;
+    }
+    candidateSet.AppendElement(CapabilityCandidate(capability));
   }
 
+  NS_DispatchToMainThread(NS_NewRunnableFunction(
+      "MediaEngineRemoteVideoSource::ChooseCapability",
+      [capabilities = mTrackCapabilities, maxHeight, minHeight, maxWidth,
+       minWidth, maxFps]() mutable {
+        dom::ULongRange widthRange;
+        widthRange.mMax.Construct(maxWidth);
+        widthRange.mMin.Construct(minWidth);
+        capabilities->mWidth.Reset();
+        capabilities->mWidth.Construct(widthRange);
+
+        dom::ULongRange heightRange;
+        heightRange.mMax.Construct(maxHeight);
+        heightRange.mMin.Construct(minHeight);
+        capabilities->mHeight.Reset();
+        capabilities->mHeight.Construct(heightRange);
+
+        dom::DoubleRange frameRateRange;
+        frameRateRange.mMax.Construct(maxFps);
+        frameRateRange.mMin.Construct(0);
+        capabilities->mFrameRate.Reset();
+        capabilities->mFrameRate.Construct(frameRateRange);
+      }));
+
   if (mCapabilitiesAreHardcoded && mCapEngine == camera::CameraEngine) {
     // We have a hardcoded capability, which means this camera didn't report
     // discrete capabilities. It might still allow a ranged capability, so we
@@ -808,13 +839,13 @@ bool MediaEngineRemoteVideoSource::ChooseCapability(
       int32_t prefHeight = aPrefs.GetHeight(isHd);
 
       cap.width = c.mWidth.Get(prefWidth);
-      cap.width = std::max(0, std::min(cap.width, 7680));
+      cap.width = std::clamp(cap.width, 0, 7680);
 
       cap.height = c.mHeight.Get(prefHeight);
-      cap.height = std::max(0, std::min(cap.height, 4320));
+      cap.height = std::clamp(cap.height, 0, 4320);
 
       cap.maxFPS = c.mFrameRate.Get(aPrefs.mFPS);
-      cap.maxFPS = std::max(0, std::min(cap.maxFPS, 480));
+      cap.maxFPS = std::clamp(cap.maxFPS, 0, 480);
 
       if (cap.width != prefWidth) {
         // Width was affected by constraints.
@@ -907,4 +938,9 @@ void MediaEngineRemoteVideoSource::GetSettings(
   aOutSettings = *mSettings;
 }
 
+void MediaEngineRemoteVideoSource::GetCapabilities(
+    dom::MediaTrackCapabilities& aOutCapabilities) const {
+  aOutCapabilities = *mTrackCapabilities;
+}
+
 }  // namespace mozilla
