@@ -41,6 +41,7 @@ class SdpAttribute {
     kDtlsMessageAttribute,
     kEndOfCandidatesAttribute,
     kExtmapAttribute,
+    kExtmapAllowMixedAttribute,
     kFingerprintAttribute,
     kFmtpAttribute,
     kGroupAttribute,
@@ -1113,6 +1114,7 @@ class SdpRtpmapAttributeList : public SdpAttribute {
     kiLBC,
     kiSAC,
     kH264,
+    kAV1,
     kRed,
     kUlpfec,
     kTelephoneEvent,
@@ -1194,6 +1196,9 @@ inline std::ostream& operator<<(std::ostream& os,
     case SdpRtpmapAttributeList::kH264:
       os << "H264";
       break;
+    case SdpRtpmapAttributeList::kAV1:
+      os << "AV1";
+      break;
     case SdpRtpmapAttributeList::kRed:
       os << "red";
       break;
@@ -1230,13 +1235,13 @@ class SdpFmtpAttributeList : public SdpAttribute {
 
     virtual ~Parameters() = default;
     virtual Parameters* Clone() const = 0;
+    virtual bool ShouldSerialize() const { return true; }
     virtual void Serialize(std::ostream& os) const = 0;
     virtual bool CompareEq(const Parameters& other) const = 0;
 
     bool operator==(const Parameters& other) const {
       return codec_type == other.codec_type && CompareEq(other);
     }
-
     SdpRtpmapAttributeList::CodecType codec_type;
   };
 
@@ -1261,6 +1266,63 @@ class SdpFmtpAttributeList : public SdpAttribute {
     std::vector<uint8_t> encodings;
   };
 
+  struct Av1Parameters : public Parameters {
+    // https://aomediacodec.github.io/av1-rtp-spec/#722-rid-restrictions-mapping-for-av1
+    Maybe<uint8_t> profile;
+    static constexpr uint8_t kDefaultProfile = 0;
+    Maybe<uint8_t> levelIdx;
+    static constexpr uint8_t kDefaultLevelIdx = 5;
+    Maybe<uint8_t> tier;
+    static constexpr uint8_t kDefaultTier = 0;
+
+    Av1Parameters() : Parameters(SdpRtpmapAttributeList::kAV1) {}
+    Av1Parameters(const Av1Parameters&) = default;
+
+    virtual ~Av1Parameters() = default;
+
+    virtual Parameters* Clone() const override {
+      return new Av1Parameters(*this);
+    }
+
+    // Returns the profile parameter if set, or the spec mandated default of 0.
+    auto profileValue() const -> uint8_t {
+      return profile.valueOr(kDefaultProfile);
+    }
+    // Returns the level-idx parameter if set, or the spec mandated default of
+    // 5.
+    auto levelIdxValue() const -> uint8_t {
+      return levelIdx.valueOr(kDefaultLevelIdx);
+    }
+    // Returns the tier parameter if set, or the spec mandated default of 0.
+    auto tierValue() const -> uint8_t { return tier.valueOr(kDefaultTier); }
+
+    virtual bool ShouldSerialize() const override {
+      return profile.isSome() || levelIdx.isSome() || tier.isSome();
+    };
+
+    virtual void Serialize(std::ostream& os) const override {
+      bool first = true;
+      profile.apply([&](const auto& profileV) {
+        os << "profile=" << static_cast<int>(profileV);
+        first = false;
+      });
+      levelIdx.apply([&](const auto& levelIdxV) {
+        os << (first ? "" : ";") << "level-idx=" << static_cast<int>(levelIdxV);
+        first = false;
+      });
+      tier.apply([&](const auto& tierV) {
+        os << (first ? "" : ";") << "tier=" << static_cast<int>(tierV);
+      });
+    }
+
+    virtual bool CompareEq(const Parameters& aOther) const override {
+      return aOther.codec_type == codec_type &&
+             static_cast<const Av1Parameters&>(aOther).profile == profile &&
+             static_cast<const Av1Parameters&>(aOther).levelIdx == levelIdx &&
+             static_cast<const Av1Parameters&>(aOther).tier == tier;
+    }
+  };
+
   class RtxParameters : public Parameters {
    public:
     uint8_t apt = 255;  // Valid payload types are 0 - 127, use 255 to represent
